#include "StdAfx.h"
#include "RawParser.h"
#include "TiffParserException.h"
#include "TiffParser.h"

/*
    RawSpeed - RAW file decoder.

    Copyright (C) 2009 Klaus Post

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

    http://www.klauspost.com
*/

namespace RawSpeed {


RawParser::RawParser(FileMap* inputData): mInput(inputData) {
}


RawParser::~RawParser(void) {
}

RawDecoder* RawParser::getDecoder() {
  const unsigned char* data = mInput->getData(0);
  if (mInput->getSize() <=  88 + 4)
    ThrowRDE("File too small");
  if (0 == memcmp(&data[0], "FUJIFILM", 8)) {
    uint32 first_ifd = data[87] | (data[86]<<8) | (data[85]<<16) | (data[84]<<24);
    first_ifd += 12;
    if (mInput->getSize() <=  first_ifd)
      ThrowRDE("File too small");
    uint32 second_ifd = (uint32)data[103] | (data[102]<<8) | (data[101]<<16) | (data[100]<<24);
    if (mInput->getSize() <=  second_ifd)
      ThrowRDE("File too small");

    try {
      FileMap m1(mInput->getDataWrt(first_ifd), mInput->getSize()-first_ifd);
      TiffParser p(&m1);
      FileMap m2(mInput->getDataWrt(second_ifd), mInput->getSize()-second_ifd);
      TiffParser p2(&m2);
      p.parseData();
      p2.parseData();
      p.MergeIFD(&p2);
      return p.getDecoder();
    } catch (TiffParserException) {}
    ThrowRDE("No decoder found. Sorry.");
  }

  try {
    TiffParser p(mInput);
    p.parseData();
    return p.getDecoder();
  } catch (TiffParserException) {}
  throw RawDecoderException("No decoder found. Sorry.");
  return NULL;
}

} // namespace RawSpeed
